/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * test_mqueue.c
 * -------------
 *
 * Test the inter-thread message queues
 */

#include "impulse/util/main.h"
#include "impulse/util/thread.h"
#include "impulse/util/event.h"

#include <stdio.h>

#if USE_THREADS
void *receive1_messages(void *data)
{
  impulse_message_queue queue;

  queue = data;

  while (!impulse_is_quit())
    {
      char *msg;

      msg = impulse_message_queue_receive(queue);
      printf("Message received (1): %s\n", msg);
      impulse_thread_nap(4,0);
      printf(" (1 - zzz)\n");
    }

  return NULL;
}

void *receive2_messages(void *data)
{
  impulse_message_queue queue;

  queue = data;

  while (!impulse_is_quit())
    {
      char *msg;

      msg = impulse_message_queue_receive(queue);
      printf("Message received (2): %s\n", msg);
      impulse_thread_nap(1,0);
      printf(" (2 - zzz)\n");
    }

  return NULL;
}

void *send_messages(void *data)
{
  impulse_message_queue queue;
  impulse_thread       thread1, thread2;
  int x;

  queue  = impulse_message_queue_create();

  for (x=0;x<10;x++)
    impulse_message_queue_send(queue, (void *) "Hello");

  thread1 = impulse_thread_create(receive1_messages, queue);
  thread2 = impulse_thread_create(receive2_messages, queue);

  while (!impulse_is_quit())
  {
    impulse_thread_nap(3,0);
    impulse_message_queue_send(queue, (void *) "Wargh");
  }

  printf("Joining reader 1\n");
  impulse_thread_join(thread1, NULL);
  printf("Joining reader 2\n");
  impulse_thread_join(thread2, NULL);
  printf("All done\n");
  return NULL;
}
#endif


impulse_bool timeout(impulse_event *ev, void *data)
{
  printf("Time to get the hell out of here\n");
  impulse_quit();
  return 0;
}

impulse_int main(impulse_int argc, 
		 impulse_char *argv[])
{
#if USE_THREADS
  impulse_thread thread;
#endif
  
  impulse_init(argc, argv);
  impulse_thread_init();
  impulse_command_parse();
  
#if USE_THREADS
  thread = impulse_thread_create(send_messages, NULL);

  impulse_event_register_ticker(impulse_event_timer(),
				30,
				0,
				timeout,
				NULL);
  
  impulse_main();
  printf("Joining sender\n");
  impulse_thread_join(thread, NULL);
  printf("Bye bye\n");
#else
  printf("Ok, who's the spoon trying to run the threads test program\n");
  printf("without first enabling thread support in libImpUtil?\n");
#endif

  impulse_thread_cleanup();
  impulse_cleanup();
  return 0;
}
