/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * channel.h
 * ------------
 *
 * Communication channel protocol
 */


#ifndef __impulse_comms_channel_h
#define __impulse_comms_channel_h

#include <impulse/comms/transport.h>

/*                            ======--------======                           */

/* 
 * Fix me: define generic error return values
 */

typedef struct _impulse_channel *impulse_channel;

/*                            ======--------======                           */

/* Creation functions */
impulse_channel impulse_channel_create(impulse_transport transport);
void impulse_channel_destroy(impulse_channel channel);

/* Event functions */
impulse_int impulse_channel_handle(impulse_channel channel,
				   impulse_transport_handler func,
				   void *data);
impulse_int impulse_channel_unhandle(impulse_channel channel);


/* channel functions */
impulse_int impulse_channel_open(impulse_channel channel,
				 impulse_unistring local,
				 impulse_unistring remote,
				 impulse_int incoming);
impulse_int impulse_channel_accept(impulse_channel channel,
				   impulse_channel *incoming);
impulse_int impulse_channel_close(impulse_channel channel);
impulse_int impulse_channel_active(impulse_channel channel);

/* Data transfer functions */
impulse_int impulse_channel_pending(impulse_channel channel);
impulse_int impulse_channel_send(impulse_channel channel,
				 impulse_byte *buffer,
				 impulse_int size,
				 impulse_unistring address);
impulse_int impulse_channel_receive(impulse_channel channel,
				    impulse_byte *buffer,
				    impulse_int size,
				    impulse_unistring *address);
impulse_int impulse_channel_peek(impulse_channel channel,
				 impulse_byte *buffer,
				 impulse_int size,
				 impulse_unistring *address);

/* Address functions */
impulse_int impulse_channel_local(impulse_channel channel,
				  impulse_unistring *address);
impulse_int impulse_channel_remote(impulse_channel channel,
				   impulse_unistring *address);


#endif

/* End of file */
