/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * transport_sock_int.h
 * --------------------
 *
 * Communication transport functions
 * Sockets protocols
 */


#ifndef __impulse_comms_transport_sock_int_h
#define __impulse_comms_transport_sock_int_h

#include "impulse/comms/transport_sock.h"
#include "impulse/comms/transport_int.h"


#define IMPULSE_TRANSPORT_SOCK_TCP 0   /* Autodetect stream protocol */
#define IMPULSE_TRANSPORT_SOCK_UDP 1   /* Autodetect datagram protocol */
#define IMPULSE_TRANSPORT_SOCK_TCP4 2  /* IPv4 stream protocol */
#define IMPULSE_TRANSPORT_SOCK_UDP4 3  /* IPv4 datagram protocol */
#define IMPULSE_TRANSPORT_SOCK_TCP6 4  /* IPv6 stream protocol */
#define IMPULSE_TRANSPORT_SOCK_UDP6 5  /* IPv6 datagram protocol */
#define IMPULSE_TRANSPORT_SOCK_TCPL 6  /* Local stream protocol */
#define IMPULSE_TRANSPORT_SOCK_UDPL 7  /* Local datagram protocol */

#define IMPULSE_TRANSPORT_SOCK_MAX IMPULSE_TRANSPORT_SOCK_UDPL + 1


struct _impulse_transport_sock
{
  impulse_int handle;

  impulse_int protocol;
  
  impulse_char *node_local;
  impulse_char *node_remote;
  impulse_char *service_local;
  impulse_char *service_remote;

  impulse_transport_handler handler;
  void *edata;
};

typedef struct _impulse_transport_sock *impulse_transport_sock;

/*                            ======--------======                           */

void *impulse_transport_sock_create(void);
void *impulse_transport_sock_create_handle(impulse_int handle,
					   impulse_int protocol);
void impulse_transport_sock_destroy(void *data);

/* Event functions */
impulse_int impulse_transport_sock_handle(void *data,
					  impulse_transport_handler func,
					  void *edata);
impulse_int impulse_transport_sock_unhandle(void *data);


/* Connection functions */
impulse_int impulse_transport_sock_open(void *data,
					impulse_unistring local,
					impulse_unistring remote,
					impulse_int incoming);
impulse_int impulse_transport_sock_accept(void *data,
					  void **client);
impulse_int impulse_transport_sock_close(void *data);
impulse_int impulse_transport_sock_active(void *data);

/* Data transfer functions */
impulse_int impulse_transport_sock_pending(void *data);
impulse_int impulse_transport_sock_send(void *data,
					impulse_byte *buffer,
					impulse_uint size,
					impulse_unistring address);
impulse_int impulse_transport_sock_receive(void *data,
					   impulse_byte *buffer,
					   impulse_uint size,
					   impulse_unistring *address);
impulse_int impulse_transport_sock_peek(void *data,
					impulse_byte *buffer,
					impulse_uint size,
					impulse_unistring *address);

/* Address functions */
impulse_int impulse_transport_sock_local(void *data,
					 impulse_unistring *address);
impulse_int impulse_transport_sock_remote(void *data,
					  impulse_unistring *address);

#endif

/* End of file */
