/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the ROWS widget library
 * for Impulse.
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * bitmap.h
 * --------
 *
 * Device-independant bitmap widget
 */

#ifndef __impulse_rows_bitmap_h
#define __impulse_rows_bitmap_h

#include <impulse/rows/colour.h>
#include <impulse/util/unistring.h>

/*                            ======--------======                           */


/* Bitmap widget type */

typedef void **ROWS_bitmap;

/*                            ======--------======                           */

/* 
 * This include file is after the definition of ROWS_bitmap, cos that
 * is needed by ROWS_drawable
 */
#include <impulse/rows/drawable.h>

struct _ROWS_bitmap_handler
{
  impulse_int (*load)(ROWS_bitmap,
		      impulse_unistring);
  impulse_int (*save)(ROWS_bitmap,
		      impulse_unistring);
};

typedef struct _ROWS_bitmap_handler *ROWS_bitmap_handler;

/* Colour depths */

#define ROWS_BITMAP_MONO        1
#define ROWS_BITMAP_4COL        2
#define ROWS_BITMAP_16COL       4
#define ROWS_BITMAP_256COL      8
#define ROWS_BITMAP_HICOLOUR   16
#define ROWS_BITMAP_TRUECOLOUR 32

/* Mask types */

typedef enum _ROWS_bitmap_mask_type
{
  ROWS_BITMAP_MASK_NONE = 0,
  ROWS_BITMAP_MASK_SIMPLE,
  ROWS_BITMAP_MASK_ALPHA
} ROWS_bitmap_mask_type;

/*                            ======--------======                           */

/* Data types */

typedef struct _ROWS_bitmap_mask
{
  ROWS_drawable drawable;
  ROWS_colour   opaque;
  ROWS_colour   transparent;
} ROWS_bitmap_mask;

/*                            ======--------======                           */

/* Standard widget functions */

extern ROWS_widget  ROWS_bitmap_create(microns width,
				       microns height,
				       impulse_int coldepth,
				       ROWS_bitmap_mask_type mask);
extern ROWS_widget *ROWS_bitmap_initialise(impulse_int id,
					   microns width,
					   microns height,
					   impulse_int coldepth,
					   ROWS_bitmap_mask_type mask);
extern impulse_int ROWS_bitmap_next_identifier(void);
extern ROWS_bitmap ROWS_bitmap_cast(void **widget);

/*                            ======--------======                           */

/* Class-specific functions */

extern void          ROWS_bitmap_resize  (ROWS_bitmap bitmap,
					  microns x,
					  microns y);
extern void          ROWS_bitmap_load    (ROWS_bitmap bitmap,
					  impulse_unistring filename);
extern void          ROWS_bitmap_save    (ROWS_bitmap bitmap,
					  impulse_unistring filename);
extern ROWS_intpoint ROWS_bitmap_get_size(ROWS_bitmap bitmap);
extern impulse_uint *ROWS_bitmap_get_data(ROWS_bitmap bitmap);

extern ROWS_bitmap_mask ROWS_bitmap_get_mask(ROWS_bitmap bitmap);

extern void          ROWS_bitmap_modified  (ROWS_bitmap bitmap);
extern impulse_uint  ROWS_bitmap_get_mod_id(ROWS_bitmap bitmap);

/* Conversion functions */

extern ROWS_bitmap ROWS_bitmap_convert(ROWS_bitmap bitmap,
				       impulse_int newdepth,
				       impulse_ubyte *newpalette);
extern impulse_ubyte *ROWS_bitmap_optimise(ROWS_bitmap bitmap,
					   impulse_int newdepth,
					   impulse_int no_entries);

#endif
