/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the ROWS widget library
 * for Impulse.
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * colour.c
 * --------
 *
 * ROWS colour manipulation
 */

#include "../../../config.h"

#include "impulse/rows/drawable_int.h"
#include "impulse/util/memory.h"
#include "impulse/util/thread.h"

extern ROWS_colour ROWS_colour_create_for_drawable(void **drawable,
						   int    r,
						   int    g,
						   int    b)
{
  ROWS_colour col;
  _p_ROWS_drawable_data **private;
  ROWS_drawable_fparam param;

  ROWS_widget_lock(drawable);

  private = (_p_ROWS_drawable_data **) drawable;

  col = impulse_alloc(sizeof(struct _ROWS_colour));

  col->drawable = drawable;
  col->r    = r;
  col->g    = g;
  col->b    = b;

  param.colour.drawable = drawable;
  param.colour.data.create.r = r;
  param.colour.data.create.g = g;
  param.colour.data.create.b = b;

  ((*private)->abstraction_layer.function[ROWS_DRAWABLE_COLOUR_CREATE])
    (&param);
  col->data = param.colour.data.create.data;

  ROWS_widget_unlock(drawable);

  return col;
}

extern ROWS_colour ROWS_colour_create_pixel_colour(void **drawable,
						   impulse_int  pixel)
{
  ROWS_colour col;
  _p_ROWS_drawable_data **private;
  ROWS_drawable_fparam param;

  ROWS_widget_lock(drawable);

  private = (_p_ROWS_drawable_data **) drawable;

  col = impulse_alloc(sizeof(struct _ROWS_colour));

  col->drawable = drawable;
  col->r    = -1;
  col->g    = -1;
  col->b    = -1;

  param.colour.drawable                = drawable;
  param.colour.data.create_pixel.pixel = pixel;

  ((*private)->abstraction_layer.function[ROWS_DRAWABLE_COLOUR_PIXEL])
    (&param);
  col->data = param.colour.data.create.data;

  ROWS_widget_unlock(drawable);

  return col;
}

void ROWS_colour_destroy(ROWS_colour c)
{
  _p_ROWS_drawable_data **private;
  ROWS_drawable_fparam param;

  ROWS_widget_lock(c->drawable);

  private = (_p_ROWS_drawable_data **) c->drawable;

  param.colour.drawable          = c->drawable;
  param.colour.data.discard.data = c->data;

  ((*private)->abstraction_layer.function[ROWS_DRAWABLE_COLOUR_DISCARD])
    (&param);

  ROWS_widget_unlock(c->drawable);

  impulse_free(c);
}


/* End of file */
