/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * iterator.c
 * ----------
 *
 * Collection iterators
 */

#include "../../../config.h"

#include "impulse/util/memory.h"
#include "impulse/util/iterator.h"

/*                            ======--------======                           */

struct _impulse_iterator
{
  impulse_iterator_funcs funcs;
  void *data;
};

/*                            ======--------======                           */

impulse_iterator impulse_iterator_create(impulse_iterator_funcs funcs,
					 void *data)
{
  impulse_iterator iterator = impulse_alloc(sizeof(struct _impulse_iterator));
  
  iterator->funcs = funcs;
  iterator->data = data;
  
  return iterator;
}


void impulse_iterator_destroy(impulse_iterator iterator)
{
  (*iterator->funcs->destroy)(iterator->data);
  impulse_free(iterator);
}


impulse_bool impulse_iterator_eof(impulse_iterator iterator)
{
  return (*iterator->funcs->eof)(iterator->data);
}


void impulse_iterator_reset(impulse_iterator iterator)
{
  (*iterator->funcs->reset)(iterator->data);
}


void *impulse_iterator_next(impulse_iterator iterator)
{
  return (*iterator->funcs->next)(iterator->data);
}


/* End of file */
