/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * stack.h
 * -------
 *
 * Double headed stacks
 */

#ifndef __impulse_util_stack_h
#define __impulse_util_stack_h

#include <impulse/util/iterator.h>

#ifdef __cplusplus
extern "C" {  
#endif

/*                            ======--------======                           */

/* stacking data structures */

typedef struct _impulse_stack *impulse_stack;
typedef void (*impulse_stack_cleaner_func)(void *data);

/*                            ======--------======                           */

/* stacking functions */

impulse_stack impulse_stack_create(void);
void impulse_stack_destroy(impulse_stack stack);

void impulse_stack_cleaner(impulse_stack stack,
			   impulse_stack_cleaner_func func);
impulse_iterator impulse_stack_elements(impulse_stack stack);

impulse_bool impulse_stack_empty(impulse_stack stack);
impulse_uint impulse_stack_count(impulse_stack stack);

void impulse_stack_push(impulse_stack stack,
			       void *data);
void *impulse_stack_pop(impulse_stack stack);
void *impulse_stack_peek(impulse_stack stack);
void impulse_stack_remove(impulse_stack stack,
			  void *data);
void impulse_stack_clear(impulse_stack stack);
impulse_bool impulse_stack_contains(impulse_stack stack,
				    void *data);

#ifdef __cplusplus
}
#endif

#endif

