/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * unichar.h
 * ---------
 *
 * Data structures to represent, and functions to manipulate, Unicode characters.
 */

#ifndef __impulse_util_unichar_h
#define __impulse_util_unichar_h
    
#include <impulse/util/setup.h>

#ifdef __cplusplus
extern "C" {  
#endif

/*                            ======--------======                           */

/* ctype.h equivalent functions */

/*
 * Under Unicode, some of these properties are informative, as they
 * may not apply to certain scripts. Note that the extra parameters
 * are for surrogate characters.
 *
 * Return values:
 *  0 - False
 *  1 - True
 * -1 - Unknown/Doesn't apply
 *
 * Other values are reserved
 */

/* FIXME: extend this to cover other script properties where necessary */

int impulse_unicode_isalnum (impulse_unichar c, ...); /* Informative */
int impulse_unicode_isalpha (impulse_unichar c, ...); /* Informative */
int impulse_unicode_isascii (impulse_unichar c, ...); /* Informative */
int impulse_unicode_iscntrl (impulse_unichar c, ...); /* Normative   */
int impulse_unicode_isdigit (impulse_unichar c, ...); /* Normative   */
int impulse_unicode_isgraph (impulse_unichar c, ...); /* Informative */
int impulse_unicode_islower (impulse_unichar c, ...); /* Informative */
int impulse_unicode_isprint (impulse_unichar c, ...); /* Informative */
int impulse_unicode_ispunct (impulse_unichar c, ...); /* Informative */
int impulse_unicode_isspace (impulse_unichar c, ...); /* Normative   */
int impulse_unicode_isupper (impulse_unichar c, ...); /* Informative */
int impulse_unicode_isxdigit(impulse_unichar c, ...); /* Informative */

/* Extensions */

int impulse_unicode_isnumber  (impulse_unichar c, ...); /* Normative   */
int impulse_unicode_isnumeric (impulse_unichar c, ...); /* Normative   */
int impulse_unicode_isprivate (impulse_unichar c, ...); /* Normative   */
int impulse_unicode_isbreak   (impulse_unichar c, ...); /* Normative   */
int impulse_unicode_isassigned(impulse_unichar c, ...); /* Normative   */
int impulse_unicode_istitle   (impulse_unichar c, ...); /* Informative */
int impulse_unicode_ismark    (impulse_unichar c, ...); /* Normative   */
int impulse_unicode_issmark   (impulse_unichar c, ...); /* Normative   */
int impulse_unicode_isnsmark  (impulse_unichar c, ...); /* Normative   */

/* Bidirectional properties */

int impulse_unicode_islr      (impulse_unichar c, ...); /* Normative   */
int impulse_unicode_isneutral (impulse_unichar c, ...); /* Normative   */
int impulse_unicode_isseparatr(impulse_unichar c, ...); /* Normative   */

/* Conversion functions */

/*
 * Under Unicode, these are *informative*. They do not apply to some
 * character sets. You may have noted they cannot apply to characters
 * represented by a surrogate pair.
 */

impulse_unichar impulse_unicode_toascii(impulse_unichar c);
impulse_unichar impulse_unicode_tolower(impulse_unichar c);
impulse_unichar impulse_unicode_toupper(impulse_unichar c);

/*                            ======--------======                           */

/* Locale functions */

/* Hmm, we need a list of locale functions */

#ifdef __cplusplus
}
#endif

#endif
